"""JARVIS KickCog."""
from jarvis_core.db.models import Kick
from naff import InteractionContext, Permissions
from naff.models.discord.embed import EmbedField
from naff.models.discord.user import User
from naff.models.naff.application_commands import (
    OptionTypes,
    slash_command,
    slash_option,
)
from naff.models.naff.command import check

from jarvis.utils import build_embed
from jarvis.utils.cogs import ModcaseCog
from jarvis.utils.permissions import admin_or_permissions


class KickCog(ModcaseCog):
    """JARVIS KickCog."""

    @slash_command(name="kick", description="Kick a user")
    @slash_option(name="user", description="User to kick", opt_type=OptionTypes.USER, required=True)
    @slash_option(
        name="reason", description="Kick reason", opt_type=OptionTypes.STRING, required=True
    )
    @check(admin_or_permissions(Permissions.BAN_MEMBERS))
    async def _kick(self, ctx: InteractionContext, user: User, reason: str) -> None:
        if not user or user == ctx.author:
            await ctx.send("You cannot kick yourself.", ephemeral=True)
            return
        if user == self.bot.user:
            await ctx.send("I'm afraid I can't let you do that", ephemeral=True)
            return
        if len(reason) > 100:
            await ctx.send("Reason must be < 100 characters", ephemeral=True)
            return

        guild_name = ctx.guild.name
        embed = build_embed(
            title=f"You have been kicked from {guild_name}",
            description=f"Reason: {reason}",
            fields=[],
        )

        embed.set_author(
            name=ctx.author.username + "#" + ctx.author.discriminator,
            icon_url=ctx.author.display_avatar.url,
        )
        embed.set_thumbnail(url=ctx.guild.icon.url)

        send_failed = False
        try:
            await user.send(embed=embed)
        except Exception:
            send_failed = True
        try:
            await ctx.guild.kick(user, reason=reason)
        except Exception as e:
            await ctx.send(f"Failed to kick user:\n```\n{e}\n```", ephemeral=True)
            return

        fields = [EmbedField(name="DM Sent?", value=str(not send_failed))]
        embed = build_embed(
            title="User Kicked",
            description=f"Reason: {reason}",
            fields=fields,
        )

        embed.set_author(name=user.display_name, icon_url=user.display_avatar.url)
        embed.set_thumbnail(url=user.display_avatar.url)
        embed.set_footer(text=f"{user.username}#{user.discriminator} | {user.id}")

        k = Kick(
            user=user.id,
            reason=reason,
            admin=ctx.author.id,
            guild=ctx.guild.id,
        )
        await k.commit()
        await ctx.send(embed=embed)
